@@ -1,14 +1,14 @@
-# @(#)$Id: Storage.pm 368 2012-04-17 18:54:37Z pjf $
+# @(#)$Id: Storage.pm 380 2012-05-19 21:01:16Z pjf $
 
 package File::DataClass::Storage;
 
 use strict;
 use namespace::autoclean;
-use version; our $VERSION = qv( sprintf '0.9.%d', q$Rev: 368 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.10.%d', q$Rev: 380 $ =~ /\d+/gmx );
 
 use Moose;
 use Class::Null;
-use English qw(-no_match_vars);
+use English     qw(-no_match_vars);
 use File::Copy;
 use File::DataClass::Constants;
 use File::DataClass::HashMerge;
@@ -19,8 +19,8 @@ with qw(File::DataClass::Util);
 
 has 'backup' => is => 'ro', isa => 'Str',    default  => NUL;
 has 'extn'   => is => 'ro', isa => 'Str',    default  => NUL;
-has 'schema' => is => 'ro', isa => 'Object', required => 1, weak_ref => TRUE,
-   handles   => { _cache => q(cache), _debug => q(debug), _lock  => q(lock),
+has 'schema' => is => 'ro', isa => 'Object', required => TRUE, weak_ref => TRUE,
+   handles   => { _cache => q(cache), _debug => q(debug), _lock => q(lock),
                   _log   => q(log),   _perms => q(perms) };
 
 sub delete {
@@ -52,6 +52,11 @@ sub dump {
       $self->_write_file( $path, $data, TRUE ) } );
 }
 
+sub extensions {
+   return { '.json' => [ q(JSON) ],
+            '.xml'  => [ q(XML::Simple), q(XML::Bare) ], };
+}
+
 sub insert {
    my ($self, $path, $result) = @_;
 
@@ -65,21 +70,11 @@ sub load {
       and return ($self->_read_file( $paths[ 0 ], FALSE ))[ 0 ] || {};
 
    my ($data, $meta, $newest) = $self->_cache->get_by_paths( \@paths );
+   my $cache_mtime  = $self->_meta_unpack( $meta );
 
-   not $self->_is_stale( $data, $meta, $newest ) and return $data;
+   not $self->is_stale( $data, $cache_mtime, $newest ) and return $data;
 
-   $data = {}; $newest = 0;
-
-   for my $path (@paths) {
-      my ($red, $path_mtime) = $self->_read_file( $path, FALSE );
-
-      $red or next; $path_mtime > $newest and $newest = $path_mtime;
-
-      for (keys %{ $red }) {
-         $data->{ $_ } = exists $data->{ $_ }
-                       ? merge( $data->{ $_ }, $red->{ $_ } ) : $red->{ $_ };
-      }
-   }
+   ($data, $newest) = $self->_load( \@paths );
 
    $self->_cache->set_by_paths( \@paths, $data, $self->_meta_pack( $newest ) );
 
@@ -173,14 +168,29 @@ sub _create_or_update {
    return $updated;
 }
 
-sub _is_stale {
-   my ($self, $data, $meta, $path_mtime) = @_;
+sub _load {
+   my ($self, $paths) = @_; my $data = {}; my $newest = 0;
 
-   my $cache_mtime = $self->_meta_unpack( $meta );
+   for my $path (@{ $paths }) {
+      my ($red, $path_mtime) = $self->_read_file( $path, FALSE ); $red or next;
 
-   return ! defined $data || ! defined $path_mtime || ! defined $cache_mtime
-         || $path_mtime > $cache_mtime
-          ? TRUE : FALSE;
+      $path_mtime > $newest and $newest = $path_mtime;
+      $self->_merge_hash_data( $data, $red );
+   }
+
+   return ($data, $newest);
+}
+
+sub _merge_hash_data {
+   my ($self, $existing, $new) = @_;
+
+   for (keys %{ $new }) {
+      $existing->{ $_ } = exists $existing->{ $_ }
+                        ? merge( $existing->{ $_ }, $new->{ $_ } )
+                        : $new->{ $_ };
+   }
+
+   return;
 }
 
 sub _meta_pack {
@@ -199,19 +209,21 @@ sub _read_file {
    $self->_lock->set( k => $path ); my ($data, $meta, $path_mtime);
 
    try {
-      ($data, $meta) = $self->_cache->get( $path );
-      $path_mtime    = $path->stat->{mtime};
+      ($data, $meta)  = $self->_cache->get( $path );
+      $path_mtime     = $path->stat->{mtime};
 
-      if ($self->_is_stale( $data, $meta, $path_mtime ) ) {
+      my $cache_mtime = $self->_meta_unpack( $meta );
+
+      if ($self->is_stale( $data, $cache_mtime, $path_mtime )) {
          if ($for_update and not $path->is_file) { $data = undef }
          else {
             $data = inner( $path->lock ); $path->close;
             $meta = $self->_meta_pack( $path_mtime );
             $self->_cache->set( $path, $data, $meta );
-            $self->_debug and $self->_log->debug( "Read file  $path" );
+            $self->_debug and $self->_log->debug( "Read file  ${path}" );
          }
       }
-      else { $self->_debug and $self->_log->debug( "Read cache $path" ) }
+      else { $self->_debug and $self->_log->debug( "Read cache ${path}" ) }
    }
    catch { $self->_lock->reset( k => $path ); $self->throw( $_ ) };
 
@@ -237,7 +249,7 @@ sub _write_file {
       catch { $path->delete; $self->throw( $_ ) };
 
       $self->_cache->remove( $path );
-      $self->_debug and $self->_log->debug( "Write file $path" )
+      $self->_debug and $self->_log->debug( "Write file ${path}" )
    }
    catch { $self->_lock->reset( k => $path ); $self->throw( $_ ) };
 
@@ -271,7 +283,7 @@ File::DataClass::Storage - Storage base class
 
 =head1 Version
 
-0.9.$Revision: 368 $
+0.10.$Revision: 380 $
 
 =head1 Synopsis
 
@@ -295,6 +307,14 @@ an error otherwise. Path is an instance of L<File::DataClass::IO>
 Dumps the data to the specified path. Path is an instance of
 L<File::DataClass::IO>
 
+=head2 extensions
+
+   $hash_ref = $storage->extensions;
+
+Returns a hash ref whose keys are the supported extensions and whose values
+are an array ref of storage subclasses that implement reading/writing files
+with that extension
+
 =head2 insert
 
    $bool = $storage->insert( $path, $result );
@@ -329,6 +349,12 @@ an error otherwise. Path is an instance of L<File::DataClass::IO>
 
 =head2 validate_params
 
+=head2 _merge_hash_data
+
+   $self->_merge_hash_data( $existsing, $new );
+
+Uses L<Hash::Merge> to merge data from the new hash ref in with the existsing
+
 =head1 Diagnostics
 
 None
@@ -341,10 +367,10 @@ None
 
 =over 3
 
-=item L<File::DataClass::Base>
-
 =item L<File::DataClass::HashMerge>
 
+=item L<File::DataClass::Util>
+
 =item L<Hash::Merge>
 
 =item L<Scalar::Util>
@@ -367,7 +393,7 @@ Peter Flanigan, C<< <Support at RoxSoft.co.uk> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2009 Peter Flanigan. All rights reserved
+Copyright (c) 2012 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
