@@ -1,16 +1,16 @@
-# @(#)$Id: XML.pm 368 2012-04-17 18:54:37Z pjf $
+# @(#)$Id: XML.pm 380 2012-05-19 21:01:16Z pjf $
 
 package File::DataClass::Storage::XML;
 
 use strict;
 use namespace::autoclean;
-use version; our $VERSION = qv( sprintf '0.9.%d', q$Rev: 368 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.10.%d', q$Rev: 380 $ =~ /\d+/gmx );
 
+use Moose;
 use File::DataClass::Constants;
 use File::DataClass::Constraints qw(HashRefOfBools);
 use MooseX::Types::Moose qw(ArrayRef Str);
 use XML::DTD;
-use Moose;
 
 extends qw(File::DataClass::Storage);
 
@@ -25,7 +25,7 @@ has '_dtd'      => is => 'rw', isa => ArrayRef,       default => sub { [] },
    init_arg     => 'dtd';
 
 around '_meta_pack' => sub {
-   my ($orig, $self, $args) = @_; my $packed = $self->$orig( $args );
+   my ($next, $self, $args) = @_; my $packed = $self->$next( $args );
 
    $self->_dtd and $packed->{_dtd} = $self->_dtd;
 
@@ -33,11 +33,11 @@ around '_meta_pack' => sub {
 };
 
 around '_meta_unpack' => sub {
-   my ($orig, $self, $packed) = @_; $packed ||= {};
+   my ($next, $self, $packed) = @_; $packed ||= {};
 
    $self->_dtd( exists $packed->{_dtd} ? delete $packed->{_dtd} : [] );
 
-   return $self->$orig( $packed );
+   return $self->$next( $packed );
 };
 
 # Private methods
@@ -115,7 +115,7 @@ File::DataClass::Storage::XML - Read/write XML data storage model
 
 =head1 Version
 
-0.9.$Revision: 368 $
+0.10.$Revision: 380 $
 
 =head1 Synopsis
 
@@ -144,9 +144,7 @@ None
 
 =item L<File::DataClass::Storage>
 
-=item L<Hash::Merge>
-
-=item L<List::Util>
+=item L<XML::DTD>
 
 =back
 
@@ -166,7 +164,7 @@ Peter Flanigan, C<< <Support at RoxSoft.co.uk> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2009 Peter Flanigan. All rights reserved
+Copyright (c) 2012 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
